require 'spec_helper'
require 'nexaas/cipher/id_crypter'

describe Nexaas::Cipher::IDCrypter do
  context 'normal behavior without salt' do
    let(:crypter) { described_class.new('') }

    describe '#salt' do
      it { expect(crypter.salt?).to be(false) }
      it { expect(crypter.salt.salt).to be_empty }
    end

    describe '#encrypt' do
      it { expect(crypter.encrypt(nil)).to be(nil) }
      it { expect(crypter.encrypt('dGVzdA==')).to eq('dGVzdA==') }
    end

    describe '#decrypt' do
      it { expect(crypter.decrypt(nil)).to be(nil) }
      it { expect(crypter.decrypt('dGVzdA==')).to eq('dGVzdA==') }
    end
  end

  context 'normal behavior with salt' do
    let(:crypter) { described_class.new(salt) }
    let(:salt) { 'iIl7wbZsP7Gd2WphNoo/64sdhwwrzEBh' }

    describe '#salt' do
      it { expect(crypter.salt?).to be(true) }
      it { expect(crypter.salt.salt).to eq(salt) }
    end

    describe '#encrypt' do
      it { expect(crypter.encrypt(nil)).to be(nil) }
      it { expect(crypter.encrypt('dGVzdA==')).to eq('HSwfQw==') }

      it 'should not obfuscate the salt' do
        expect(crypter.encrypt('dGVzdDE=')).to eq(crypter.encrypt('dGVzdDE='))
      end
    end

    describe '#decrypt' do
      it { expect(crypter.decrypt(nil)).to be(nil) }
      it { expect(crypter.decrypt('dGVzdA==')).to eq('HSwfQw==') }

      it 'should not obfuscate the salt' do
        expect(crypter.decrypt('dGVzdDE=')).to eq(crypter.decrypt('dGVzdDE='))
      end
    end
  end

  context 'altered behavior without salt' do
    let(:crypter) { described_class.new('') }

    describe '#encrypt' do
      it { expect(crypter.encrypt(nil, base64: false)).to be(nil) }
      it { expect(crypter.encrypt('test', base64: false)).to eq('dGVzdA==') }
    end

    describe '#decrypt' do
      it { expect(crypter.decrypt(nil, base64: false)).to be(nil) }
      it { expect(crypter.decrypt('dGVzdA==', base64: false)).to eq('test') }
    end
  end

  context 'altered behavior with salt' do
    let(:crypter) { described_class.new(salt) }
    let(:salt) { 'iIl7wbZsP7Gd2WphNoo/64sdhwwrzEBh' }

    describe '#encrypt' do
      it { expect(crypter.encrypt(nil, base64: false)).to be(nil) }
      it { expect(crypter.encrypt('test', base64: false)).to eq('HSwfQw==') }

      it 'should not obfuscate the salt' do
        expect(crypter.encrypt('test1', base64: false)).
          to eq(crypter.encrypt('test1', base64: false))
      end
    end

    describe '#decrypt' do
      it { expect(crypter.decrypt(nil, base64: false)).to be(nil) }
      it { expect(crypter.decrypt('HSwfQw==', base64: false)).to eq('test') }

      it 'should not obfuscate the salt' do
        expect(crypter.decrypt('dGVzdDE=', base64: false)).
          to eq(crypter.decrypt('dGVzdDE=', base64: false))
      end
    end
  end
end
