require 'spec_helper'

describe Nexaas::Cipher::Salt do
  context 'empty salt' do
    subject { described_class.new('') }

    it { expect(subject.salt?).to be(false) }
    it { expect(subject.salt).to be_empty }
    it { expect(subject.salt('test')).to eq("\0\0\0\0") }
    it { expect(subject.salt('test', base64: true)).to eq('AAAAAA==') }
  end

  context 'non-coded' do
    subject { described_class.new('abc') }

    it { expect(subject.salt?).to be(true) }
    it { expect(subject.salt).to eq('abc') }

    context 'full length' do
      it { expect(subject.salt('te')).to eq('abc') }
      it { expect(subject.salt('tes')).to eq('abc') }
      it { expect(subject.salt('testtest')).to eq('abcabcab') }
      it { expect(subject.salt('testtest', base64: true)).to eq('YWJjYWJjYWI=') }
    end

    context 'truncated' do
      it { expect(subject.salt('te', truncate: true)).to eq('ab') }
      it { expect(subject.salt('tes', truncate: true)).to eq('abc') }
      it { expect(subject.salt('testtest', truncate: true)).to eq('abcabcab') }
      it { expect(subject.salt('te', base64: true, truncate: true)).to eq('YWI=') }
    end

    context 'coded' do
      subject { described_class.new('ZGVm', base64: true) }

      it { expect(subject.salt?).to be(true) }
      it { expect(subject.salt).to eq('def') }

      context 'full length' do
        it { expect(subject.salt('te')).to eq('def') }
        it { expect(subject.salt('tes')).to eq('def') }
        it { expect(subject.salt('testtest')).to eq('defdefde') }
        it { expect(subject.salt('testtest', base64: true)).to eq('ZGVmZGVmZGU=') }
      end

      context 'truncated' do
        it { expect(subject.salt('te', truncate: true)).to eq('de') }
        it { expect(subject.salt('tes', truncate: true)).to eq('def') }
        it { expect(subject.salt('testtest', truncate: true)).to eq('defdefde') }
        it { expect(subject.salt('te', base64: true, truncate: true)).to eq('ZGU=') }
      end
    end
  end
end
