require 'spec_helper'
require 'nexaas/cipher/xor_crypter'

describe Nexaas::Cipher::XorCrypter do
  let(:crypter) { described_class.new(salt, base64: true) }

  context 'without salt' do
    let(:salt) { '' }

    it { expect(crypter.salt?).to be(false) }
    it { expect(crypter.encrypt(nil)).to be(nil) }
    it { expect(crypter.encrypt('test')).to eq('test') }
    it { expect(crypter.encrypt('test', trucate: true)).to eq('test') }
    it { expect(crypter.encrypt('test', ofuscate: true)).to eq('test') }
    it { expect(crypter.encrypt('AA==', base64: true)).to eq('AA==') }
  end

  context 'with salt' do
    let(:salt) { '0LhN0KgMsNksgZQmfYCjweg68cRxCr9086IzcqM1f6w=' }

    describe '#salt' do
      it { expect(crypter.salt?).to be(true) }
      it { expect(crypter.salt.salt(base64: true)).to eq(salt) }
    end

    describe '#encrypt' do
      it { expect(crypter.encrypt(nil)).to be(nil) }

      it 'should truncate' do
        expect(crypter.encrypt('test1', truncate: true)).
        to eq("\xa4\xdd>\xa4\x99".force_encoding('ASCII-8BIT'))
      end

      it 'should not obfuscate the salt' do
        expect(crypter.encrypt('test1')).to eq(crypter.encrypt('test1'))
      end

      it 'should obfuscate the salt' do
        expect(crypter.encrypt('test1', obfuscate: true)).
          not_to eq(crypter.encrypt('test1', obfuscate: true))
      end
    end

    describe '#decrypt' do
      it { expect(crypter.decrypt(crypter.encrypt('test1'))).to eq('test1') }
      it { expect(crypter.decrypt(crypter.encrypt('test1', truncate: true))).to match(/\Atest1/) }
      it { expect(crypter.decrypt(crypter.encrypt('test1', obfuscate: true))).to eq('test1') }
    end
  end
end
